// Modified from https://www.shadertoy.com/view/MtByRh

precision highp float;

varying vec2 fragCoord;
uniform vec3 iResolution;
uniform sampler2D iChannel0;
uniform float iTime;
uniform float iAlpha;

////////////////////////////////////////////////////////////////////////////////
//lensing.fsh
// This content is under the MIT License.

#define iterations 12
#define formuparam 0.57

#define volsteps 10
#define stepsize 0.2

#define zoom   1.200
#define tile   1.0
#define speed  0.010

#define brightness 0.0015
#define darkmatter 1.00
#define def_distfading 0.730
#define saturation 1.0

//#define mo (2.0 * iMouse.xy - iResolution.xy) / iResolution.y
#define blackholeCenter vec3(time*2.,time,-2.)
#define def_blackholeRadius 0.5
#define blackholeIntensity 1.0

float iSphere(vec3 ray, vec3 dir, vec3 center, float radius)
{
	vec3 rc = ray-center;
	float c = dot(rc, rc) - (radius*radius);
	float b = dot(dir, rc);
	float d = b*b - c;
	float t = -b - sqrt(abs(d));
	float st = step(0.0, min(t,d));
	return mix(-1.0, t, st);
}

vec3 iPlane(vec3 ro, vec3 rd, vec3 po, vec3 pd){
    float d = dot(po - ro, pd) / dot(rd, pd);
    return d * rd + ro;
}

vec3 r(vec3 v, vec2 r)//incomplete but ultrafast rotation fcn thnx to rodolphito
{
    vec4 t = sin(vec4(r, r + 1.5707963268));
    float g = dot(v.yz, t.yw);
    return vec3(v.x * t.z - g * t.x,
                v.y * t.w - v.z * t.y,
                v.x * t.x + g * t.z);
}

float sampleMusicA() {
/*
	return 0.5 * (
		texture2D( iChannel0, vec2( 0.15, 0.25 ) ).x +
		texture2D( iChannel0, vec2( 0.30, 0.25 ) ).x);
*/
	return 1.0 / 3.0 * (
//		texture2D( iChannel0, vec2( 100.0/22050.0, 0.25 ) ).x
		texture2D( iChannel0, vec2( 500.0/22050.0, 0.25 ) ).x
		+ texture2D( iChannel0, vec2( 2000.0/22050.0, 0.25 ) ).x
		+ texture2D( iChannel0, vec2( 4000.0/22050.0, 0.25 ) ).x
//		+ texture2D( iChannel0, vec2( 6000.0/22050.0, 0.25 ) ).x
		);
}

float sampleMusicB() {
	return 1.0 / 2.0 * (
		texture2D( iChannel0, vec2( 100.0/22050.0, 0.25 ) ).x
//		+ texture2D( iChannel0, vec2( 500.0/22050.0, 0.25 ) ).x
//		+ texture2D( iChannel0, vec2( 2000.0/22050.0, 0.25 ) ).x
//		+ texture2D( iChannel0, vec2( 4000.0/22050.0, 0.25 ) ).x
		+ texture2D( iChannel0, vec2( 6000.0/22050.0, 0.25 ) ).x
		);
}

float sampleMusic() {
	return 1.0 / 5.0 * (
		texture2D( iChannel0, vec2( 100.0/22050.0, 0.25 ) ).x
		+ texture2D( iChannel0, vec2( 500.0/22050.0, 0.25 ) ).x
		+ texture2D( iChannel0, vec2( 2000.0/22050.0, 0.25 ) ).x
		+ texture2D( iChannel0, vec2( 4000.0/22050.0, 0.25 ) ).x
		+ texture2D( iChannel0, vec2( 6000.0/22050.0, 0.25 ) ).x
		);
}
//lensing.fsh
////////////////////////////////////////////////////////////////////////////////


#define PI 3.1415926
#define PI2 6.2831852

#define hue(h)clamp(abs(fract(h + vec4(3, 2, 1, 0) / 3.0) * 6.0 - 3.0) - 1.0 , 0.0, 1.0)


void main()
{
    vec4 outColor;

	////////////////////////////////////////////////////////////////////////////////
	//lensing.fsh
	{
        //get coords and direction
        vec2 uv=fragCoord.xy-.5;
        if (iResolution.x > iResolution.y)
            uv.x *= iResolution.x / iResolution.y;
        else
            uv.y *= iResolution.y / iResolution.x;

        vec3 dir=vec3(uv*zoom,1.);
        float time=iTime*speed+.25;

        //mouse rotation
        vec3 from=vec3(0.0, 0.0, -15.0);
        //from = r(from, mo / 10.0);
        //dir = r(dir, mo / 10.0);
        from+=blackholeCenter;

        vec3 nml = normalize(blackholeCenter - from);
        vec3 pos = iPlane(from, dir, blackholeCenter, nml);
        pos = blackholeCenter - pos;

        float intensity = dot(pos, pos) * 0.4; //0.5 for bigger blackhole
        float pulse = sampleMusic();
        intensity *= -0.5*pulse*1.5 + 1.0;
        //intensity += -0.2*sampleMusicB();

        float distfading = def_distfading;
        distfading *= pulse*0.8 + 0.4;

        float blackholeRadius = def_blackholeRadius;
        //blackholeRadius += sampleMusicB();

        if(intensity > blackholeRadius * blackholeRadius){
            intensity = 1.0 / intensity;
            dir = mix(dir, pos * sqrt(intensity), blackholeIntensity * intensity);

            //volumetric rendering
            float s=0.1,fade=1.;
            vec3 v=vec3(0.);
            for (int r=0; r<volsteps; r++) {
                vec3 p=from+s*dir*.5;
                p = abs(vec3(tile)-mod(p,vec3(tile*2.))); // tiling fold
                float pa,a=pa=0.;
                for (int i=0; i<iterations; i++) {
                    p=abs(p)/dot(p,p)-formuparam; // the magic formula
                    a+=abs(length(p)-pa); // absolute sum of average change
                    pa=length(p);
                }
                float dm=max(0.,darkmatter-a*a*.001); //dark matter
                a*=a*a; // add contrast
                if (r>6) fade*=1.-dm; // dark matter, don't render near
                //v+=vec3(dm,dm*.5,0.);
                v+=fade;
                v+=vec3(s,s*s,s*s*s*s)*a*brightness*fade; // coloring based on distance
                fade*=distfading; // distance fading
                s+=stepsize;
            }
            v=mix(vec3(length(v)),v,saturation); //color adjust
            outColor = vec4(v*.01, 1.0);
        }
        else outColor = vec4(0.0);
    }

	{
        vec2 uv = (fragCoord - 0.5); //(fragCoord - iResolution.xy * 0.5) / iResolution.y;
        if (iResolution.x > iResolution.y)
            uv.x *= iResolution.x / iResolution.y;
        else
            uv.y *= iResolution.y / iResolution.x;


        float CIRCLES = 15.0;
        float cS = 0.375;

        float sm = 1.0 / iResolution.y * 2.0; // smooth
        float ps = 1.0 / iResolution.y * sqrt(iResolution.y) * 0.225; // circle thin

        float d = length(uv);

        float a = atan(uv.y, uv.x);
        a = a < 0.0 ? PI + (PI - abs(a)) : a;

        float lPos = a /PI2;

        float m = 0.0;
        float partSize = 1.0 / CIRCLES;
        vec3 col;
        for(float i = CIRCLES; i > 1.0; i -= 1.0) {

            float ilPos = fract(lPos + i*0.1 + iTime * 0.1);
            float cPos = partSize * i + ilPos * partSize;
            float invPos = partSize * (i + 1.0) - ilPos * partSize;
            //float nzF = (1.0 - ilPos);
            float mP0 = texture2D(iChannel0, vec2(partSize * i, 0.0)).x;
            float mP = texture2D(iChannel0, vec2(cPos, 0.0)).x;
            float mPInv = texture2D(iChannel0, vec2(invPos, 0.0)).x;

            mP = (mP + mPInv) / 2.0;

            float rDiff = i*(1.0 / CIRCLES * 0.35);
            float r = 0.4 * mP * (1.0 / CIRCLES * 3.0) - rDiff; //slowhand: 0.4 for bigger circle

            float subm = smoothstep(cS - ps + r, cS - ps + sm + r, d) * smoothstep(cS + r, cS - sm + r, d);

            if (subm > 0.0) {
                col = hue(i / CIRCLES * 0.5 + iTime * 0.05 + mP0 * 0.84).rgb;
            }

            m += subm;
        }

        m = clamp(m, 0.0, 1.0);

        float r = (sin(iTime * 0.5) * 0.5 + 0.5);
        float b = (cos(iTime * 0.5) * 0.5 + 0.5);
        vec3 backCol = vec3(r, 0.0, b) * length(uv * 0.75) * 0.5;

        col = mix(backCol, col, m);

        outColor += vec4(col, 1.0);
	}

	gl_FragColor = outColor;
}